@@ -55,16 +55,19 @@ public:
       std::string load_forest_filename, ImportanceMode importance_mode, uint min_node_size,
       std::string split_select_weights_file, std::vector<std::string>& always_split_variable_names,
       std::string status_variable_name, bool sample_with_replacement,
-      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule);
+      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
+      std::string case_weights_file, bool predict_all, double sample_fraction);
   void initR(std::string dependent_variable_name, Data* input_data, uint mtry, uint num_trees,
       std::ostream* verbose_out, uint seed, uint num_threads, ImportanceMode importance_mode, uint min_node_size,
-      std::vector<double>& split_select_weights, std::vector<std::string>& always_split_variable_names,
+      std::vector<std::vector<double>>& split_select_weights, std::vector<std::string>& always_split_variable_names,
       std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
-      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule);
+      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
+      std::vector<double>& case_weights, bool predict_all, bool keep_inbag, double sample_fraction);
   void init(std::string dependent_variable_name, MemoryMode memory_mode, Data* input_data, uint mtry,
       std::string output_prefix, uint num_trees, uint seed, uint num_threads, ImportanceMode importance_mode,
       uint min_node_size, std::string status_variable_name, bool prediction_mode, bool sample_with_replacement,
-      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule);
+      std::vector<std::string>& unordered_variable_names, bool memory_saving_splitting, SplitRule splitrule,
+      bool predict_all, double sample_fraction);
   virtual void initInternal(std::string status_variable_name) = 0;
 
   // Grow or predict
@@ -134,6 +137,14 @@ public:
     return is_ordered_variable;
   }
 
+  std::vector<std::vector<size_t>> getInbagCounts() const {
+    std::vector<std::vector<size_t>> result;
+    for (auto& tree : trees) {
+      result.push_back(tree->getInbagCounts());
+    }
+    return result;
+  }
+
 protected:
   void grow();
   virtual void growInternal() = 0;
@@ -157,7 +168,7 @@ protected:
   virtual void loadFromFileInternal(std::ifstream& infile) = 0;
 
   // Set split select weights and variables to be always considered for splitting
-  void setSplitWeightVector(std::vector<double>& split_select_weights);
+  void setSplitWeightVector(std::vector<std::vector<double>>& split_select_weights);
   void setAlwaysSplitVariables(std::vector<std::string>& always_split_variable_names);
 
   // Show progress every few seconds
@@ -183,6 +194,9 @@ protected:
   bool sample_with_replacement;
   bool memory_saving_splitting;
   SplitRule splitrule;
+  bool predict_all;
+  bool keep_inbag;
+  double sample_fraction;
 
   // For each varID true if ordered
   std::vector<bool> is_ordered_variable;
@@ -208,7 +222,10 @@ protected:
   // Deterministic variables are always selected
   std::vector<size_t> deterministic_varIDs;
   std::vector<size_t> split_select_varIDs;
-  std::vector<double> split_select_weights;
+  std::vector<std::vector<double>> split_select_weights;
+
+  // Bootstrap weights
+  std::vector<double> case_weights;
 
   // Random number generator
   std::mt19937_64 random_number_generator;
